#define UART_NUMBER 1 // Promini
#define CURRENTPORT 0 // Promini

#define RX_BUFFER_SIZE 256 // 256 RX buffer is needed for GPS communication (64 or 128 was too short)
#define TX_BUFFER_SIZE 128
#define INBUF_SIZE 64
#define BIND_CAPABLE 0;  //Used for Spektrum today; can be used in the future for any RX type that needs a bind and has a MultiWii module. 

static volatile uint8_t serialHeadRX[UART_NUMBER],serialTailRX[UART_NUMBER];
static uint8_t serialBufferRX[RX_BUFFER_SIZE][UART_NUMBER];
static volatile uint8_t serialHeadTX[UART_NUMBER],serialTailTX[UART_NUMBER];
static uint8_t serialBufferTX[TX_BUFFER_SIZE][UART_NUMBER];
static uint8_t inBuf[INBUF_SIZE][UART_NUMBER];

//static uint8_t checksum[UART_NUMBER];

void SerialGpsPrint(prog_char* str) { // USAGE: SerialGpsPrint(PSTR("$PUBX,41,1,0003,0001,19200,0*23\r\n"));
 char b;
 while(str && (b = pgm_read_byte(str++))) SerialWrite(b);
}

// *******************************************************
// Interrupt driven UART transmitter - using a ring buffer
// *******************************************************
void serialize8(uint8_t a) {
  uint8_t t = serialHeadTX[CURRENTPORT];
  if (++t >= TX_BUFFER_SIZE) t = 0;
  serialBufferTX[t][CURRENTPORT] = a;
//  checksum[CURRENTPORT] ^= a;
  serialHeadTX[CURRENTPORT] = t;
}

  ISR(USART_UDRE_vect) {  // Serial 0 on a PROMINI
    uint8_t t = serialTailTX[0];
    if (serialHeadTX[0] != t) {
      if (++t >= TX_BUFFER_SIZE) t = 0;
      UDR0 = serialBufferTX[t][0];  // Transmit next byte in the ring
      serialTailTX[0] = t;
    }
    if (t == serialHeadTX[0]) UCSR0B &= ~(1<<UDRIE0); // Check if all data is transmitted . if yes disable transmitter UDRE interrupt
  }

void UartSendData() {
    UCSR0B |= (1<<UDRIE0);
}

bool SerialTXfree() {
    return (serialHeadTX[CURRENTPORT] == serialTailTX[CURRENTPORT]);
  }

static void inline SerialOpen(uint32_t baud) {
  uint8_t h = ((F_CPU  / 4 / baud -1) / 2) >> 8;
  uint8_t l = ((F_CPU  / 4 / baud -1) / 2);
  UCSR0A  = (1<<U2X0); UBRR0H = h; UBRR0L = l; UCSR0B |= (1<<RXEN0)|(1<<TXEN0)|(1<<RXCIE0);
}

static void SerialEnd() {
 UCSR0B &= ~((1<<RXEN0)|(1<<TXEN0)|(1<<RXCIE0)|(1<<UDRIE0));
}

static void inline store_uart_in_buf(uint8_t data, uint8_t portnum) {
  uint8_t h = serialHeadRX[portnum];
  if (++h >= RX_BUFFER_SIZE) h = 0;
  if (h == serialTailRX[portnum]) return; // we did not bite our own tail?
  serialBufferRX[serialHeadRX[portnum]][portnum] = data;  
  serialHeadRX[portnum] = h;
}

ISR(USART_RX_vect)  { store_uart_in_buf(UDR0, 0); }

uint8_t SerialRead() {
  uint8_t t = serialTailRX[CURRENTPORT];
  uint8_t c = serialBufferRX[t][CURRENTPORT];
  if (serialHeadRX[CURRENTPORT] != t) {
    if (++t >= RX_BUFFER_SIZE) t = 0;
    serialTailRX[CURRENTPORT] = t;
  }
  return c;
}

uint8_t SerialAvailable() {
//uint8_t SerialAvailable(uint8_t port) {
  return (serialHeadRX[CURRENTPORT] - serialTailRX[CURRENTPORT])%RX_BUFFER_SIZE;
}

void SerialWrite(uint8_t c){
  serialize8(c);UartSendData();
}

